@@ -2,8 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-pref("startup.homepage_override_url", "https://www.waterfox.net/blog/waterfox-%DISPLAY_VERSION%-release/?update");
-pref("startup.homepage_welcome_url", "https://www.waterfox.net/blog/waterfox-%DISPLAY_VERSION%-release/?new");
+pref("startup.homepage_override_url", "https://github.com/WaterfoxCo/Waterfox-Classic/releases/tag/%DISPLAY_VERSION%-classic");
+pref("startup.homepage_welcome_url", "https://github.com/WaterfoxCo/Waterfox-Classic/releases/tag/%DISPLAY_VERSION%-classic");
 pref("startup.homepage_welcome_url.additional", "");
 // Interval: Time between checks for a new version (in seconds)
 pref("app.update.interval", 43200); // 12 hours
@@ -15,12 +15,12 @@ pref("app.update.download.backgroundInterval", 0);
 pref("app.update.promptWaitTime", 691200);
 // URL user can browse to manually if for some reason all update installation
 // attempts fail.
-pref("app.update.url.manual", "https://www.waterfox.net/download/");
+pref("app.update.url.manual", "https://github.com/WaterfoxCo/Waterfox-Classic/releases/latest");
 // A default value for the "More information about this update" link
 // supplied in the "An update is available" page of the update wizard.
 pref("app.update.url.details", "https://www.waterfox.net/blog/");
 
-pref("app.releaseNotesURL", "https://www.waterfox.net/blog/waterfox-%DISPLAY_VERSION%-release/");
+pref("app.releaseNotesURL", "https://github.com/WaterfoxCo/Waterfox-Classic/releases/tag/%DISPLAY_VERSION%-classic");
 
 // The number of days a binary is permitted to be old
 // without checking for an update.  This assumes that
