@@ -1817,6 +1817,16 @@ MediaDecoder::RequestDebugInfo()
     });
 }
 
+void
+MediaDecoder::GetMozDebugReaderData(nsACString& aString)
+{
+  aString += nsPrintfCString("Container Type: %s\n",
+                             ContainerType().Type().AsString().get());
+  if (mReader) {
+    mReader->GetMozDebugReaderData(aString);
+  }
+}
+
 void
 MediaDecoder::NotifyAudibleStateChanged()
 {
