@@ -108,9 +108,6 @@ public:
 
   bool SupportsConfig(IMFMediaType* aType, float aFramerate) override;
 
-  bool CreateDXVA2Decoder(const VideoInfo& aVideoInfo,
-                          nsACString& aFailureReason) override;
-
 private:
   bool CanCreateDecoder(const DXVA2_VideoDesc& aDesc,
                         const float aFramerate) const;
@@ -507,31 +504,6 @@ DXVA2Manager::CreateD3D9DXVA(layers::KnowsCompositor* aKnowsCompositor,
   return nullptr;
 }
 
-bool
-D3D9DXVA2Manager::CreateDXVA2Decoder(const VideoInfo& aVideoInfo,
-                                     nsACString& aFailureReason)
-{
-  MOZ_ASSERT(NS_IsMainThread());
-  DXVA2_VideoDesc desc;
-  desc.SampleWidth = aVideoInfo.mImage.width;
-  desc.SampleHeight = aVideoInfo.mImage.height;
-  desc.Format = (D3DFORMAT)MAKEFOURCC('N','V','1','2');
-
-  // Assume the current duration is representative for the entire video.
-  float framerate = 1000000.0 / aVideoInfo.mDuration.ToMicroseconds();
-  if (IsUnsupportedResolution(desc.SampleWidth, desc.SampleHeight, framerate)) {
-    return false;
-  }
-
-  mDecoder = CreateDecoder(desc);
-  if (!mDecoder) {
-    aFailureReason =
-      nsPrintfCString("Fail to create video decoder in D3D9DXVA2Manager.");
-    return false;
-  }
-  return true;
-}
-
 bool
 D3D9DXVA2Manager::CanCreateDecoder(const DXVA2_VideoDesc& aDesc,
                                    const float aFramerate) const
@@ -622,9 +594,6 @@ public:
 
   bool SupportsConfig(IMFMediaType* aType, float aFramerate) override;
 
-  bool CreateDXVA2Decoder(const VideoInfo& aVideoInfo,
-                          nsACString& aFailureReason) override;
-
 private:
   HRESULT CreateFormatConverter();
 
@@ -1111,32 +1080,6 @@ D3D11DXVA2Manager::ConfigureForSize(uint32_t aWidth, uint32_t aHeight)
   return S_OK;
 }
 
-bool
-D3D11DXVA2Manager::CreateDXVA2Decoder(const VideoInfo& aVideoInfo,
-                                      nsACString& aFailureReason)
-{
-  MOZ_ASSERT(NS_IsMainThread());
-  D3D11_VIDEO_DECODER_DESC desc;
-  desc.Guid = mDecoderGUID;
-  desc.OutputFormat = DXGI_FORMAT_NV12;
-  desc.SampleWidth = aVideoInfo.mImage.width;
-  desc.SampleHeight = aVideoInfo.mImage.height;
-
-  // Assume the current duration is representative for the entire video.
-  float framerate = 1000000.0 / aVideoInfo.mDuration.ToMicroseconds();
-  if (IsUnsupportedResolution(desc.SampleWidth, desc.SampleHeight, framerate)) {
-    return false;
-  }
-
-  mDecoder = CreateDecoder(desc);
-  if (!mDecoder) {
-    aFailureReason =
-      nsPrintfCString("Fail to create video decoder in D3D11DXVA2Manager.");
-    return false;
-  }
-  return true;
-}
-
 bool
 D3D11DXVA2Manager::CanCreateDecoder(const D3D11_VIDEO_DECODER_DESC& aDesc,
                                     const float aFramerate) const
