@@ -7,9 +7,11 @@
 #include "AudioDestinationNode.h"
 #include "AlignmentUtils.h"
 #include "AudioContext.h"
+#include "mozilla/dom/AudioContextBinding.h"
 #include "mozilla/dom/AudioDestinationNodeBinding.h"
 #include "mozilla/dom/OfflineAudioCompletionEvent.h"
 #include "mozilla/dom/ScriptSettings.h"
+#include "mozilla/dom/BaseAudioContextBinding.h"
 #include "mozilla/Services.h"
 #include "AudioChannelAgent.h"
 #include "AudioChannelService.h"
